(function($){
	
	$.Class.extend('UndoRedoStack', {
		
			// current record
			current: -1,
			// history limit
			limit: 100,
			// name of stack
			name: "",
		
		init: function()
		{
			// functions stack
			this.stack = [];
		},
		
		clear: function()
		{
			this.stack = [];
			this.current = -1;
		},
		
		push: function(func)
		{
			if(this.stack.length > this.limit) {
				// remove first element
				this.stack.shift();
			}
			this.stack.push( func || null );
		},
		
		pop: function()
		{
			var func = this.stack[this.current] || null;
			if(func) {
				func();
			}
		}
		
	});
	
	$.fg.UndoRedo = new ($.Class.extend({
		
			_undo: new UndoRedoStack(),
			_redo: new UndoRedoStack(),
			
			undoChar: 'z',
			redoChar: 'y',
		
		init: function()
		{
			$(document).keydown( $.proxy( function(e) {
				if (e.ctrlKey && !e.shiftKey && e.which == this._charCode( this.undoChar )) {
					this.undo();
				}
				if (e.ctrlKey && !e.shiftKey && e.which == this._charCode( this.redoChar )) {
					this.redo();
				}
			}, this ) );
		},
		
		push: function(undo, redo)
		{
			this._undo.push( undo );
			this._redo.push( redo );
			
			this._undo.current = this._redo.stack.length-1;
			this._redo.current = this._redo.stack.length;
			
			this.triggerOnChange();
		},
		
		undo: function()
		{
			if(!this.canUndo()) {
				return;
			}
			this._undo.pop();
			
			this._undo.current--;
			this._redo.current--;
			
			this.triggerOnChange();
		},
		
		redo: function()
		{
			if(!this.canRedo()) {
				return;
			}
			this._redo.pop();
			
			this._redo.current++;
			this._undo.current++;
			
			this.triggerOnChange();
		},
		
		canUndo: function()
		{
			return this._undo.current > -1;
		},
		
		canRedo: function()
		{
			return this._redo.stack.length > 0 && this._redo.current < this._redo.stack.length;
		},
		
		_charCode: function(character)
		{
			return character.toUpperCase().charCodeAt();
		},
		
		triggerOnChange: function()
		{
			$(document).trigger( "fg.undoredo.change" );
		}
		
	}));
	
})(jQuery);

/**
 * @desc wrapper function for $.fg.UndoRedo.push()
 */
function undoRedo(undo, redo)
{
	$.fg.UndoRedo.push( undo, redo );
}