<?php
/**
 * Class for making compatible YII locale and date format with formats of other extensions.
 */
class LocaleHelper
{
    /**
     * @var array map of yii date format to jquery-ui date format
     */
    private static $_yiiToJq = array(
//        'D' => 'o',not used
//        'DDD' => 'oo',
        'EEE' => 'D',
        'EEEE' => 'DD',
        'cccc' => 'DD',
        'M' => 'm',
        'MM' => 'mm',
        'MMM' => 'M',
        'MMMM' => 'MM',
        'y' => 'yy',
        'yy' => 'y',
        'yyyy' => 'yy'
    );

    /**
     * @var array jQuery locales
     */
    private static $_jqLocales = array('af', 'ar', 'az', 'bg', 'bs', 'ca', 'cs', 'da', 'de-CH', 'de', 'el', 'en-GB', 'eo', 'es', 'et', 'eu', 'fa', 'fi', 'fo', 'fr-CH', 'fr', 'he', 'hr', 'hu', 'hy', 'id', 'is', 'it', 'ja', 'ko', 'lt', 'lv', 'ms', 'nl-BE', 'nl', 'no', 'pl', 'pt-BR', 'ro', 'ru', 'sk', 'sl', 'sq', 'sr-SR', 'sr', 'sv', 'ta', 'th', 'tr', 'uk', 'vi', 'zh-CN', 'zh-HK', 'zh-TW');

    /**
     * @var array map of Yii locales to jQuery locales
     */
    private static $_yiiToJqLocaleMap = array(
        'sr_cyrl' => 'sr',
        'sr_latn' => 'sr_sr'
    );

    /**
     * @static
     * @param $format YII date format
     * @return mixed JQuery date format
     */
    public static function yiiFormatToJq($format)
    {
        return self::convert(self::$_yiiToJq, $format);
    }

    private static function convert($substitution, $format)
    {
        if (empty($format)) return false;
        $i = 0;
        $finished = false;
        while (!$finished) {
            $singleFormat = '';
            $prevL = $format[$i];
            $end = false;
            while (!$end) {
                if ($prevL != @$format[$i])
                    $end = true;
                else {
                    $singleFormat .= $format[$i];
                    $i++;
                }
            }
            if (isset($substitution[$singleFormat])) {
                $format = substr_replace($format, $substitution[$singleFormat], $i - strlen($singleFormat), strlen($singleFormat));
                $i += strlen($substitution[$singleFormat]) - strlen($singleFormat);
            }
            if (@$format[$i] === '')
                $finished = true;
        }
        return $format;
    }

    /**
     * @static
     * @param $locale YII locale
     * @return string jQuery locale
     */
    public static function yiiLocaleToJq($locale)
    {
        if (empty($locale)) return false;
        $parts = explode('_', $locale);
        if ($parts[0] == 'en')
            return 'en-GB';
        elseif (count($parts) == 1)
            return $locale;
        elseif (count($parts) == 3) {
            $locale = $parts[0] . '_' . $parts[1];
        }
        if (isset(self::$_yiiToJqLocaleMap[$locale]))
            $locale = self::$_yiiToJqLocaleMap[$locale];

        $parts = explode('_', $locale);
        $jqLocale = $parts[0];
        $jqExtLocale = isset($parts[1]) ? $parts[0] . '-' . strtoupper($parts[1]) : $parts[0];
        if (in_array($jqExtLocale, self::$_jqLocales))
            return $jqExtLocale;
        elseif (in_array($jqLocale, self::$_jqLocales))
            return $jqLocale;
        else
            return 'en-GB';

    }
}
